# Laravel Squire Client by Myriad

## Install

### Add Myriad Repository

The client is distributed internally with Satis. First add the repository to `composer.json`:

```json
  "repositories": [
    {
      "type": "composer",
      "url": "https://satis.myriadmobile.com"
    }
  ]
```

### Require the Dependency

Require this package with composer using the following command:

```bash
composer require myriad/squire-client
```

### Install Service Provider

After updating composer, add the service provider to the `providers` array in `config/app.php`:

```php
Myriad\Squire\SquireServiceProvider::class,
```

### Install Facade (Optional)

If you choose, register the facade in the `aliases` array in `config/app.php`:

```php
'Squire' => Myriad\Squire\Facades\Squire::class,
```

### Configuration

By default, the following environment variables are read to setup Squire:

* `SQUIRE_TOKEN` => The application's unique server token
* `SQUIRE_SERVER` => The url to the Squire server. Default: https://squire.myriadapps.com

If environment variables are not sufficient, the configuration file can be published using:

```bash
php artisan vendor:publish --provider=myriad/squire-client --tag=config
```

## Usage

Note. Examples make use of the Facade. For best practice, inject an instance of `Myriad\Squire\Contracts\Squire`.

### Quick Start

Upload a file from a request and get the url:

```php
<?php namespace App\Http\Controllers;

// from https://laravel.com/docs/5.5/filesystem#file-uploads

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class UserAvatarController extends Controller {
  
    /**
     * Update the avatar for the user.
     *
     * @param  Request  $request
     * @return Response
     */
    public function update(Request $request) {
      
        $path = Squire::generateUniquePathForFile($request->file('avatar'));
        Squire::putFile($path, $request->file('avatar'));
        $url = Squire::file($path)->build();

        return $url;
    }
}
```

Upload a local file and print the url:

```php
<?php namespace App\Console\Commands;

use Illuminate\Console\Command;

/**
 * Class UploadImage uploads an image to squire and returns the URL.
 * @package App\Console\Commands
 */
class UploadImage extends Command {

  protected $signature = 'image:upload {filename}';
  
  public function handle() {
    
    $filename = $this->argument('filename');

    $fileDescriptor = new \SplFileInfo($filename);

    $path = Squire::generateUniquePathForFile($fileDescriptor);
    $url = Squire::file($path)->build();
    Squire::putFile($path, $fileDescriptor);

    $this->line($url);
  }
}
```

### URLs

#### Files

To get the url for file, use the file builder:

```php
$url = Squire::file($path)->build();
```

`$path` is an identifier assigned to a file, most often this is a filename. e.g. `myfile.pdf`. Paths can also contain
 slashes to help organize your files in to folders: `myfolder/myfile.pdf`.

#### Images

To get the url for an image, use the image builder:

```php
$url = Squire::image($path)->width(200)->grayscale()->build();
```

__NOTE: To be processed correctly by the Glide sub-processor, the image path must end with a known image file extension
or send an image Content-Type.__

### Get File

Squire makes retrieving your files painless:

```php
$resource = Squire::getAsStream($path);
```

or as a string:

```php
$string = Squire::getAsString($path);
```

### Put File

Pushing a Laravel UploadedFile to Squire is as easy as:

```php
Squire::putUploadedFile(string $path, UploadedFile $file);
```

or a regular php file:

```php
Squire::putFile(string $path, SplFileInfo $file, string $contentType = 'binary/octet-stream');
```

or any resource stream:

```php
Squire::putStream(string $path, $stream, int $contentLength, string $contentType = 'binary/octet-stream');
```

__NOTE: It is possible to replace a source file, however, files already in the cache will not reflect the changes. 
For this reason, it's recommended to generate a new path each time a file is uploaded and delete the previous file.__

### Delete File

Deleting a file is simple:

```php
Squire::delete($path);
```

__NOTE: Currently only source files are deleted, this means cached files will continue to be served until they expire.__

### Helpers

Generate a random path name with the specified file extension. Optionally, a prefix may be specified.

```php
Squire::generateUniquePathForExtension($extension, $prefix = '');
```

Generate a random path name for the given file. Optionally, a prefix may be specified.

```php
Squire::generateUniquePathForFile(\SplFileInfo $file, $prefix = '');
```
