<?php namespace Myriad\Squire\Contracts;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Psr7\Stream;
use Illuminate\Http\UploadedFile;
use Myriad\Squire\FileResponse;
use Myriad\Squire\SquireConfiguration;
use Myriad\Squire\SquireFile;
use Myriad\Squire\SquireImage;
use SplFileInfo;

interface Squire {

  /**
   * Create a new instance of squire
   *
   * @param \Myriad\Squire\SquireConfiguration $config
   * @param \GuzzleHttp\Client $guzzle
   */
  public function __construct(SquireConfiguration $config, Guzzle $guzzle);

  /**
   * Get the data for the given path
   *
   * @param string $path
   * @return FileResponse
   */
  public function get(string $path);

  /**
   * Get the file as a stream resource
   *
   * @param string $path
   * @return resource
   */
  public function getAsStream(string $path);

  /**
   * @param string $path
   * @return GuzzleHttp\Psr7\Stream
   */
  public function getAsGuzzleStream(string $path);

  /**
   * Get the file as string
   *
   * @param string $path
   * @return string
   */
  public function getAsString(string $path);

  /**
   * Put the UploadedFile
   *
   * @param string $path
   * @param UploadedFile $file
   * @return FileResponse
   */
  public function putUploadedFile(string $path, UploadedFile $file);

  /**
   * Put the SplFileInfo
   *
   * @param string $path
   * @param SplFileInfo $file
   * @param string|null $contentType the mime type. e.g. image/png
   * @return FileResponse
   */
  public function putFile(string $path, SplFileInfo $file, string $contentType = 'binary/octet-stream');

  /**
   * Put the stream
   *
   * @param string $path
   * @param resource $stream
   * @param int $contentLength file size in bytes
   * @param string $contentType the mime type. e.g. image/png
   * @return FileResponse
   */
  public function putStream(string $path, $stream, int $contentLength, string $contentType = 'binary/octet-stream');

  /**
   * Delete the file
   *
   * @param string $path
   * @return boolean
   */
  public function delete(string $path);

  /**
   * Make a SquireImage
   *
   * @param string $path the file path
   *
   * @return SquireImage
   */
  public function image($path);

  /**
   * Make a SquireFile
   *
   * @param string $path the file path
   *
   * @return SquireFile
   */
  public function file($path);

  /**
   * Get the Squire configuration
   *
   * @return SquireConfiguration
   */
  public function getConfiguration();

  /**
   * Generate a unique, cryptographically secure, path for a file
   *
   * @param SplFileInfo $file
   * @param string $prefix
   * @return mixed
   */
  public function generateUniquePathForFile(\SplFileInfo $file, $prefix = '');

  /**
   * Generate a unique, cryptographically secure, path with the given file extension
   *
   * @param string $extension
   * @param string $prefix
   * @return mixed
   */
  public function generateUniquePathForExtension($extension, $prefix = '');

}