<?php namespace Myriad\Squire;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Psr7\Stream;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Str;
use Myriad\Squire\Contracts\Squire;
use SplFileInfo;

class SquireClient implements Squire {

  /**
   * @var \Myriad\Squire\SquireConfiguration
   */
  private $config;

  /**
   * @var \GuzzleHttp\Client
   */
  private $guzzle;

  /**
   * @inheritdoc
   */
  public function __construct(SquireConfiguration $config, Guzzle $guzzle) {
    $this->config = $config;
    $this->guzzle = $guzzle;
  }

  /**
   * @inheritdoc
   */
  public function get(string $path) {
    $response = $this->doFileRequest('GET', ['query' => ['path' => $path]]);
    return new FileResponse(json_decode($response->getBody(), true));
  }

  /**
   * @inheritdoc
   */
  public function getAsStream(string $path) {
    $response = $this->doFileRequest('GET', ['query' => ['path' => $path, 'alt' => 'media']]);
    return $response->getBody()->detach();
  }

  /**
   * @param string $path
   * @return Stream
   */
  public function getAsGuzzleStream(string $path) {
    return new Stream($this->getAsStream($path));
  }

  /**
   * @inheritdoc
   */
  public function getAsString(string $path) {
    $response = $this->doFileRequest('GET', ['query' => ['path' => $path, 'alt' => 'media']]);
    return $response->getBody()->getContents();
  }

  /**
   * @inheritdoc
   */
  public function putUploadedFile(string $path, UploadedFile $file) {
    return $this->putStream($path, fopen($file->getRealPath(), 'r'), $file->getClientSize(), $file->getClientMimeType());
  }

  /**
   * @inheritdoc
   */
  public function putFile(string $path, SplFileInfo $file, string $contentType = 'binary/octet-stream') {
    return $this->putStream($path, fopen($file->getRealPath(), 'r'), $file->getSize(), $contentType);
  }

  /**
   * @inheritdoc
   */
  public function putStream(string $path, $stream, int $contentLength, string $contentType = 'binary/octet-stream') {
    $response = $this->doFileRequest('POST', [
      'query' => ['path' => $path],
      'body' => $stream,
      'headers' => [
        'Content-Type' => $contentType,
        'Content-Length' => $contentLength,
      ],
    ]);
    return new FileResponse(json_decode($response->getBody(), true));
  }

  /**
   * @inheritdoc
   */
  public function delete(string $path) {
    $this->doFileRequest('DELETE', ['query' => ['path' => $path]]);
    return true;
  }

  /**
   * @inheritdoc
   */
  public function image($path) {
    return new SquireImage($path, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function file($path) {
    return new SquireFile($path, $this->getConfiguration());
  }

  /**
   * @inheritdoc
   */
  public function getConfiguration() {
    return $this->config;
  }

  /**
   * @inheritdoc
   */
  public function generateUniquePathForFile(\SplFileInfo $file, $prefix = '') {
    $extension = $file->getExtension();
    if ($file instanceof UploadedFile) {
      $extension = $file->getClientOriginalExtension();
    }
    return $this->generateUniquePathForExtension($extension, $prefix);
  }

  /**
   * @inheritdoc
   */
  public function generateUniquePathForExtension($extension, $prefix = '') {
    if (!empty($prefix) && !ends_with($prefix, '/')) {
      $prefix .= '-';
    }
    return trim($prefix . Str::random(64) . '.' . ltrim($extension, '.'), '/.');
  }

  /**
   * @inheritdoc
   */
  private function doFileRequest($method, $options = []) {
    array_set($options, 'connect_timeout', 10);
    array_set($options, 'headers.Authorization', $this->getConfiguration()->getServerToken());
    $url = $this->config->getServerUrl() . '/api/v2/files';

    try {
      return $this->guzzle->request($method, $url, $options);
    } catch (ClientException $e) {
      if ($e->getResponse()->getStatusCode() == 404) {
        throw new FileNotFoundException();
      }
      throw $e;
    }
  }
}
