<?php namespace Myriad\Squire;

class SquireImage extends SquireFile {

  /**
   * Sets the width of the image, in pixels.
   *
   * @param int $width the width in pixels
   *
   * @return $this
   */
  public function width(int $width) {
    return $this->param('w', $width);
  }

  /**
   * Sets the height of the image, in pixels.
   *
   * @param int $height the height in pixels
   *
   * @return $this
   */
  public function height(int $height) {
    return $this->param('h', $height);
  }

  /**
   * Sets the background color of the image.
   *
   * @param string $background hex background color
   * @return $this
   */
  public function background(string $background) {
    return $this->param('bg', ltrim($background, '#'));
  }

  /**
   * Sets how the image is fitted to its target dimensions.
   *
   * @param string $fit
   *
   * @return $this
   */
  public function fit(string $fit) {
    return $this->param('fit', $fit);
  }

  /**
   * Default. Resizes the image to fit within the width and height boundaries without cropping, distorting or altering
   * the aspect ratio.
   *
   * @return $this
   */
  public function fitContain() {
    return $this->fit('contain');
  }

  /**
   * Resizes the image to fit within the width and height boundaries without cropping, distorting or altering the
   * aspect ratio, and will also not increase the size of the image if it is smaller than the output size.
   *
   * @return $this
   */
  public function fitMax() {
    return $this->fit('max');
  }

  /**
   * Resizes the image to fit within the width and height boundaries without cropping or distorting the image, and the
   * remaining space is filled with the background color. The resulting image will match the constraining dimensions.
   *
   * @param string $background hex background color
   * @return $this
   */
  public function fitFill(string $background) {
    $this->background($background);
    return $this->fit('fill');
  }

  /**
   * Stretches the image to fit the constraining dimensions exactly. The resulting image will fill the dimensions, and
   * will not maintain the aspect ratio of the input image.
   *
   * @return $this
   */
  public function fitStretch() {
    return $this->fit('stretch');
  }

  /**
   * Resizes the image to fill the width and height boundaries and crops any excess image data. The resulting image
   * will match the width and height constraints without distorting the image.
   *
   * @param string $position
   * @return $this
   */
  public function fitCrop(string $position = 'center') {
    return $this->fit('crop' . '-' . $position);
  }


  /**
   * Crop the image from the top left.
   *
   * @return $this
   */
  public function fitCropTopLeft() {
    return $this->fitCrop('top-left');
  }

  /**
   * Crop the image from the top.
   *
   * @return $this
   */
  public function fitCropTop() {
    return $this->fitCrop('top');
  }

  /**
   * Crop the image from the top right.
   *
   * @return $this
   */
  public function fitCropTopRight() {
    return $this->fitCrop('top-right');
  }

  /**
   * Crop the image from the left.
   *
   * @return $this
   */
  public function fitCropLeft() {
    return $this->fitCrop('left');
  }

  /**
   * Crop the image from the center.
   *
   * @return $this
   */
  public function fitCropCenter() {
    return $this->fitCrop('center');
  }

  /**
   * Crop the image from the right.
   *
   * @return $this
   */
  public function fitCropRight() {
    return $this->fitCrop('right');
  }

  /**
   * Crop the image from the bottom left.
   *
   * @return $this
   */
  public function fitCropBottomLeft() {
    return $this->fitCrop('bottom-left');
  }

  /**
   * Crop the image from the bottom.
   *
   * @return $this
   */
  public function fitCropBottom() {
    return $this->fitCrop('bottom');
  }

  /**
   * Crop the image from the bottom right.
   *
   * @return $this
   */
  public function fitCropBottomRight() {
    return $this->fitCrop('bottom-right');
  }

  /**
   * Crops the image to specific dimensions prior to any other resize operations.
   *
   * @param int $width
   * @param int $height
   * @param int $x
   * @param int $y
   *
   * @return $this
   */
  public function crop(int $width, int $height, int $x, int $y) {
    return $this->param('crop', "$width,$height,$x,$y");
  }

  /**
   * Rotates the image.
   *
   * @param string $orientation
   *
   * @return $this
   */
  public function rotate(string $orientation) {
    return $this->param('or', $orientation);
  }

  /**
   * Automatically rotate the image based on Exif data.
   *
   * @return $this
   */
  public function rotateAuto() {
    return $this->rotate('auto');
  }

  /**
   * Rotate the image to 270 degrees.
   *
   * @return $this
   */
  public function rotateLeft() {
    return $this->rotate('270');
  }

  /**
   * Rotate the image to 90 degrees.
   *
   * @return $this
   */
  public function rotateRight() {
    return $this->rotate('90');
  }

  /**
   * Rotate the image 180 degrees.
   *
   * @return $this
   */
  public function rotateUpsideDown() {
    return $this->rotate('180');
  }

  /**
   * The device pixel ratio is used to easily convert between CSS pixels and device pixels. This makes it possible to
   * display images at the correct pixel density on a variety of devices such as Apple devices with Retina Displays
   * and Android devices. You must specify either a width, a height, or both for this parameter to work.
   * The default is 1. The maximum value that can be set is 8.
   *
   * @param int $density a value between 1 and 8
   * @return $this
   */
  public function density(int $density = 1) {
    return $this->param('dpr', $density);
  }

  /**
   * Adjusts the image brightness.
   *
   * @param int $brightness A value between -100 and +100, where 0 represents no change.
   *
   * @return $this
   */
  public function brightness(int $brightness) {
    return $this->param('bri', $brightness);
  }

  /**
   * Adjusts the image contrast.
   *
   * @param int $contrast A value between -100 and +100, where 0 represents no change.
   *
   * @return $this
   */
  public function contrast(int $contrast) {
    return $this->param('con', $contrast);
  }


  /**
   * Adjusts the image gamma.
   *
   * @param float $gamma a value between 0.1 and 9.99.
   *
   * @return $this
   */
  public function gamma(float $gamma) {
    return $this->param('gam', $gamma);
  }

  /**
   * Sharpen the image.
   *
   * @param int $sharpen a value between 0 and 100.
   *
   * @return $this
   */
  public function sharpen(int $sharpen) {
    return $this->param('sharp', $sharpen);
  }

  /**
   * Adds a blur effect to the image.
   *
   * @param int $blur a value between 0 and 100.
   *
   * @return $this
   */
  public function blur(int $blur) {
    return $this->param('blur', $blur);
  }

  /**
   * Applies a pixelation effect to the image.
   *
   * @param int $pixelate a value between 0 and 1000.
   *
   * @return $this
   */
  public function pixel(int $pixelate) {
    return $this->param('pixel', $pixelate);
  }

  /**
   * Applies a filter effect to the image..
   *
   * @param string $filter grayscale or sepia.
   *
   * @return $this
   */
  public function filter(string $filter) {
    return $this->param('filt', $filter);
  }

  /**
   * Apply a grayscale filter.
   *
   * @return $this
   */
  public function grayscale() {
    return $this->filter('grayscale');
  }

  /**
   * Apply a sepia filter.
   *
   * @return $this
   */
  public function sepia() {
    return $this->filter('sepia');
  }

  /**
   * Defines the quality of the image. Note, this will enforce jpg output.
   *
   * @param int $quality a value between 0 and 100. Default: 90.
   *
   * @return $this
   */
  public function quality(int $quality) {
    $this->jpg();

    return $this->param('q', $quality);
  }

  /**
   * Encodes the image to a specific format.
   *
   * @param string $format jpg, png, or gif. Default: jpg.
   *
   * @return $this
   */
  public function format(string $format) {
    return $this->param('fm', $format);
  }

  /**
   * Formats the image as a jpg.
   *
   * @return $this
   */
  public function jpg() {
    return $this->format('jpg');
  }

  /**
   * Formats the image as a png.
   *
   * @return $this
   */
  public function png() {
    return $this->format('png');
  }

  /**
   * Formats the image as a gif.
   *
   * @return $this
   */
  public function gif() {
    return $this->format('gif');
  }

  /**
   * Add a border to the image.
   *
   * @param int $width sets the border width in pixels, or using relative dimensions.
   * @param string $color Sets the border color.
   * @param string $method Sets how the border will be displayed. Available options: overlay, shrink, expand
   *
   * @return $this
   */
  public function border(int $width, string $color, string $method = 'overlay') {
    $color = ltrim($color, '#');
    return $this->param('border', "$width,$color,$method");
  }

}