<?php

use Mockery as m;
use Myriad\Squire\SquireConfiguration;

/**
 * Class SquireConfigurationTest
 *
 * Test the SquireConfiguration class
 */
class SquireConfigurationTest extends SquireTestCase {

  /**
   * Test that construction fails when a server token is not passed
   */
  public function testConstructWithoutToken() {
    $this->setExpectedException('\InvalidArgumentException', 'Configuration must contain server_token');
    $this->getConfiguration(['server_url' => $this->confUrl]);
  }

  /**
   * Test that construction fails when a server url is not passed
   */
  public function testConstructWithoutServerUrl() {
    $this->setExpectedException('\InvalidArgumentException', 'Configuration must contain server_url');
    $this->getConfiguration(['server_token' => $this->confToken]);
  }

  /**
   * Test that provisioning data is restored correctly from the cache
   */
  public function testProvisionFromCache() {
    $repository = m::mock('Illuminate\Contracts\Cache\Repository');
    $repository->shouldReceive('get')->andReturn($this->getMockServerConfiguration());

    $config = $this->getConfiguration($this->getBaseConfiguration(), $repository);
    $config->provision();

    $this->assertConfiguration($config);
  }

  /**
   * Test that provisioning from the server works correctly
   */
  public function testProvisionFromSever() {
    $repository = $this->getMockCacheRepository();
    $repository->shouldReceive('get')->andReturn([]);
    $repository->shouldReceive('forever')->with(SquireConfiguration::CACHE_KEY, $this->getMockServerConfiguration());

    $guzzle = $this->getMockGuzzleClient();
    $guzzle->shouldReceive('get')->with($this->confUrl . '/api/v2/provision', [
      'timeout' => 10,
      'headers' => [
        'Authorization' => $this->confToken
      ]
    ])->andReturnSelf();
    $guzzle->shouldReceive('getBody')->andReturn(json_encode(['data' => $this->getMockServerConfiguration()]));

    $config = $this->getConfiguration($this->getBaseConfiguration(), $repository, $guzzle);
    $config->provision();

    $this->assertConfiguration($config);
  }

  /**
   * Test that the configuration is lazily pulled when a configuration value is accessed
   */
  public function testLazyConfiguration() {
    $repository = $this->getMockCacheRepository();
    $repository->shouldReceive('get')->andReturn($this->getMockServerConfiguration());

    $config = $this->getConfiguration($this->getBaseConfiguration(), $repository);
    $this->assertConfiguration($config);
  }

  /**
   * Test that the configuration contains all of the correct values
   *
   * @param \Myriad\Squire\SquireConfiguration $config
   */
  private function assertConfiguration(SquireConfiguration $config) {
    $this->assertEquals($this->confToken, $config->getServerToken(), 'Configuration has incorrect server_token.');
    $this->assertEquals($this->confUrl, $config->getServerUrl(), 'Configuration has incorrect server_url.');
    $this->assertEquals($this->confPrefix, $config->getPrefix(), 'Configuration has incorrect prefix.');
    $this->assertEquals($this->confDomain, $config->getDomain(), 'Configuration has incorrect domain.');
  }

}