<?php

use Mockery as m;

/**
 * Class SquireServiceProviderTest
 *
 * Test the SquireServiceProvider class
 */
class SquireServiceProviderTest extends SquireTestCase {

  /**
   * Test that the service provider publishes a config file.
   */
  public function testProviderPublishesConfig() {
    $provider = new \Myriad\Squire\SquireServiceProvider($this->getMockedApp());

    $provider->boot();
    $published = $provider->pathsToPublish();

    $this->assertStringEndsWith('../config/squire.php', array_keys($published)[0]);
    $this->assertEquals('squire.php', array_values($published)[0]);
  }

  /**
   * Test that the service provider merges the default config file with the dev's.
   */
  public function testProviderMergesDefaultConfig() {
    $app = $this->getMockedApp();
    $app->shouldIgnoreMissing();

    $provider = \Mockery::mock('\Myriad\Squire\SquireServiceProvider[mergeConfigFrom]', [$app]);
    $provider->shouldAllowMockingProtectedMethods();
    $provider->shouldReceive('mergeConfigFrom')->andReturn();

    $provider->boot();
    $provider->register();
  }

  /**
   * Test that the service provider registers Squire.
   */
  public function testProviderRegistersSquire() {
    $app = $this->getMockedApp();

    $registers = [];
    $app->shouldReceive('singleton')->withAnyArgs()->andReturnUsing(function ($name, $callback) use (&$registers) {
      $registers[$name] = $callback;
    });
    $app->shouldReceive('alias')->withAnyArgs()->andReturnUsing(function ($abstract, $alias) use (&$registers) {
      $registers[$alias] = $abstract;
    });

    $provider = \Mockery::mock('\Myriad\Squire\SquireServiceProvider[mergeConfigFrom]', [$app]);
    $provider->shouldAllowMockingProtectedMethods();
    $provider->shouldReceive('mergeConfigFrom')->andReturn();

    $provider->boot();
    $provider->register();

    // test the squire singleton
    $this->assertArrayHasKey('squire', $registers, 'Provider must register squire.');
    $this->assertArraySubset(['Myriad\Squire\Contracts\Squire' => 'squire'], $registers, 'Provider must register Myriad\Squire\Contracts\Squire');
    $this->assertArraySubset(['Myriad\Squire\SquireClient' => 'squire'], $registers, 'Provider must register Myriad\Squire\SquireClient');

    $this->assertSingletonCallbackInstantiatesSquire($registers['squire']);
  }

  /**
   * Assert that the callback returns an instance of Squire.
   *
   * @param $callback
   */
  private function assertSingletonCallbackInstantiatesSquire($callback) {
    $app = $this->getMockedApp();
    $app['cache.store'] = $this->getMockCacheRepository();
    $squire = $callback($app);
    $this->assertInstanceOf('Myriad\Squire\SquireClient', $squire);
  }

  /**
   * Test that the service provider describes what it provides.
   */
  public function testProviderDescribesProvides() {
    $provider = new \Myriad\Squire\SquireServiceProvider($this->getMockedApp());
    $this->assertArraySubset(['squire', 'Myriad\Squire\Contracts\Squire', 'Myriad\Squire\SquireClient'], $provider->provides());
  }

  /**
   * Test that the provider loading is deferred as to not hurt performance.
   */
  public function testProviderIsDeferred() {
    $provider = new \Myriad\Squire\SquireServiceProvider($this->getMockedApp());
    $this->assertTrue($provider->isDeferred());
  }

  /**
   * Test that the facade returns the correct accessor.
   */
  public function testFacadeReturnsCorrectAccessor() {
    $facade = new \Myriad\Squire\Facades\Squire();
    $this->assertEquals('squire', $facade->getFacadeAccessor());
  }

}